// Exemplo de arquivo de descricao de cena para POV-ray
// Last edited on 2014-06-06 16:35:33 by stolfilocal

// ======================================================================
// CORES E TEXTURAS

background{ color rgb < 1, 1, 1 > }

#declare tx_plastico = 
  texture{
    pigment{ color rgb < 0.10, 0.80, 1.00 > }
    finish{ diffuse 0.8 ambient 0.1 specular 0.5 roughness 0.005 }
  }

#declare fin_poly = finish{ diffuse 0.9 ambient 0.1 }

#declare tx_tetra = 
  texture{
    pigment{ color rgb < 0.050, 0.050, 0.050 > }
    finish{ fin_poly }
  }

#declare tx_cubo =
  texture{
    pigment{ color rgb < 0.850, 0.850, 0.750 > }
    finish{ fin_poly }
  }
  
#declare tx_face = 
  texture{
    pigment{ color rgb < 0.900, 0.900, 0.500 > transmit 0.8 }
    finish{ diffuse 0.9 ambient 0.1 }
  }

#declare tx_nome = 
  texture{
    pigment{ color rgb < 0.000, 0.200, 1.000 > }
    finish{ diffuse 0.0 ambient 1.0 }
  }

#declare tx_sinais = 
  texture{
    pigment{ color rgb < 0.000, 0.200, 1.000 > }
    finish{ diffuse 0.0 ambient 1.0 }
  }
  
#include "eixos.inc"

#declare centro_cena = < 00.00, 00.00, -0.6 >;
#declare dir_camera = < 1.6, 1.0, 0.7 >;
#declare dist_camera = 30.0;
#declare raio_cena = 7.50;

#macro vertice(p, rad)
  sphere{ p, rad }
#end
  
#macro aresta(p,q,rad)
  cylinder{ p, q, rad }
#end
  
#macro face(p,q,r)
  triangle{ p, q, r }
#end

#macro oalign(obj,xa,ya)
  #local lo = min_extent(obj);
  #local ho = max_extent(obj);
  #local dx = ho.x - lo.x;
  #local dy = ho.y - lo.y;
  object{ obj translate < -lo.x - xa*dx, -lo.y - ya*dy, 0 > }
#end

#macro altext(txt, font, mag, xa, ya)
  #local obj = object{ text{ ttf concat("tt-fonts/", font, ".ttf") txt 0.001, 0.0 } scale < mag, mag, 1 > }
  object{ oalign(obj, xa, ya) }
#end

#macro rotulo(p, nome, sinais, xa,ya,dc)
  #local raw = 
    union{
      object{ altext(nome,   "arial",  0.5,  0.5, -0.1) texture{ tx_nome } }
      object{ altext(sinais, "courbd", 0.5,  0.5, +1.1) texture{ tx_sinais } }
    }
  #local rot = object{ oalign(raw, xa, ya) }
  #local az = degrees(atan2(dir_camera.y, dir_camera.x));
  #local el = degrees(atan2(dir_camera.z, vlength(< dir_camera.x, dir_camera.y, 0 >)));
  #local pos_camera = centro_cena + vnormalize(dir_camera)*dist_camera;
  #local dcam = vnormalize(pos_camera - p);
  object{ rot 
    rotate 90*z 
    rotate 90*y 
    rotate -el*y 
    rotate az*z
    translate p + dc*dcam
  }
#end
 
#macro cubo(R)
  #local p000 = R*< -1, -1, -1 >;
  #local p001 = R*< -1, -1, +1 >;
  #local p010 = R*< -1, +1, -1 >;
  #local p011 = R*< -1, +1, +1 >;

  #local p100 = R*< +1, -1, -1 >;
  #local p101 = R*< +1, -1, +1 >;
  #local p110 = R*< +1, +1, -1 >;
  #local p111 = R*< +1, +1, +1 >;
  
  #local rad = 0.015;
  
  union{
    union{ 
      object{ vertice(p000, rad) }
      object{ vertice(p001, rad) }
      object{ vertice(p010, rad) }
      object{ vertice(p011, rad) }
      object{ vertice(p100, rad) }
      object{ vertice(p101, rad) }
      object{ vertice(p110, rad) }
      object{ vertice(p111, rad) }
      
      object{ aresta(p000, p001, rad) }
      object{ aresta(p010, p011, rad) }
      object{ aresta(p100, p101, rad) }
      object{ aresta(p110, p111, rad) }
      
      object{ aresta(p000, p010, rad) }
      object{ aresta(p001, p011, rad) }
      object{ aresta(p100, p110, rad) }
      object{ aresta(p101, p111, rad) }
      
      object{ aresta(p000, p100, rad) }
      object{ aresta(p001, p101, rad) }
      object{ aresta(p010, p110, rad) }
      object{ aresta(p011, p111, rad) }
      
      texture{ tx_cubo }
     
    }
  }

#end

#macro tetra(R)
  #local pA = R*< +1, +1, -1 >;
  #local pT = R*< +1, -1, +1 >;
  #local pC = R*< -1, +1, +1 >;
  #local pG = R*< -1, -1, -1 >;
  
  #local rad = 0.05;
  
  union{
    union{
      object{ vertice(pA, rad) }
      object{ vertice(pT, rad) }
      object{ vertice(pC, rad) }
      object{ vertice(pG, rad) }
      
      object{ aresta(pA, pT, rad) } 
      object{ aresta(pA, pC, rad) } 
      object{ aresta(pA, pG, rad) } 
      object{ aresta(pT, pC, rad) } 
      object{ aresta(pT, pG, rad) }  
      object{ aresta(pC, pG, rad) }
      
      texture{ tx_tetra }
    }

    // union{
    //   object{ face(pA, pT, pC) } 
    //   object{ face(pA, pT, pG) } 
    //   object{ face(pA, pC, pG) } 
    //   object{ face(pT, pC, pG) } 
    //   
    //   texture{ tx_face }
    // }

    union{
      object{ rotulo(pA, "A","++-", +0.3, +1.2, -0.6*R) }
      object{ rotulo(pT, "T","+-+", +0.6, -0.3, 00.0) }
      object{ rotulo(pC, "C","-++", +0.4, -0.3, 00.0) }
      object{ rotulo(pG, "G","---", -0.5, +0.5, +1.5*R) }
    }
  }
#end

// object{ eixos(6) }
object{ tetra(3) }
object{ cubo(3) }

#include "camlight.inc"
#declare intens_luz = 1.00;
camlight(centro_cena, raio_cena, dir_camera, dist_camera , z, intens_luz)
